export const navigation = [
  { text: 'Intro', link: '/intro' },
  { text: 'Guide', link: '/guide/getting-started' },
  { text: "API Reference", link: "/api-reference" },
  { text: "Community", link: "/community" },
  { text: "Discussions", link: "https://github.com/orgs/nexios-labs/discussions" },
  { text: 'Team', link: 'team' },
  { text: 'Blog', link: 'https://blog.nexioslabs.com' },
  {
    text: 'v3  (Latest)',
    items: [
      { text: 'v3', link: '/guide/getting-started' },
      { text: 'v2.x.x', link: '/v2/guide/getting-started' },
      { text: 'Changelog', link: '/changelog' },
      { text: 'Contributing', link: '/community/contribution-guide' },
    ]
  }
]
